@@ -5,18 +5,28 @@ from pathlib import Path
 from functools import lru_cache
 
 from docutils.parsers.rst.directives.body import Sidebar
-from docutils import nodes
+from docutils import nodes as docutil_nodes
 from sphinx.application import Sphinx
 from sphinx.locale import get_translation
 from sphinx.util import logging
+from pydata_sphinx_theme import _get_theme_options
 
-from .header_buttons import prep_header_buttons, add_header_buttons
+from .nodes import SideNoteNode
+from .header_buttons import (
+    prep_header_buttons,
+    add_header_buttons,
+    update_sourcename,
+    update_context_with_repository_info,
+)
 from .header_buttons.launch import add_launch_buttons
+from .header_buttons.source import add_source_buttons
+from ._transforms import HandleFootnoteTransform
 
-__version__ = "0.3.2"
+__version__ = "1.0.0"
 """sphinx-book-theme version"""
 
 SPHINX_LOGGER = logging.getLogger(__name__)
+DEFAULT_LOG_TYPE = "sphinxbooktheme"
 MESSAGE_CATALOG_NAME = "booktheme"
 
 
@@ -43,7 +53,7 @@ def add_metadata_to_page(app, pagename, templatename, context, doctree):
     # Add a shortened page text to the context using the sections text
     if doctree:
         description = ""
-        for section in doctree.traverse(nodes.section):
+        for section in doctree.traverse(docutil_nodes.section):
             description += section.astext().replace("\n", " ")
         description = description[:160]
         context["page_description"] = description
@@ -55,10 +65,11 @@ def add_metadata_to_page(app, pagename, templatename, context, doctree):
     # Translations
     translation = get_translation(MESSAGE_CATALOG_NAME)
     context["translate"] = translation
-    # this is set in the html_theme
-    context["theme_search_bar_text"] = translation(
-        context.get("theme_search_bar_text", "Search the docs ...")
-    )
+
+    # If search text hasn't been manually specified, use a shorter one here
+    theme_options = _get_theme_options(app)
+    if "search_bar_text" not in theme_options:
+        context["theme_search_bar_text"] = translation("Search") + "..."
 
 
 @lru_cache(maxsize=None)
@@ -85,7 +96,7 @@ def hash_assets_for_files(assets: list, theme_static: Path, context):
             asset_sphinx_link = f"_static/{asset}"
             asset_source_path = theme_static / asset
             if not asset_source_path.exists():
-                SPHINX_LOGGER.warn(
+                SPHINX_LOGGER.warning(
                     f"Asset {asset_source_path} does not exist, not linking."
                 )
             # Find this asset in context, and update it to include the digest
@@ -109,10 +120,12 @@ def hash_html_assets(app, pagename, templatename, context, doctree):
     hash_assets_for_files(assets, get_html_theme_path() / "static", context)
 
 
-def update_thebe_config(app):
+def update_mode_thebe_config(app):
     """Update thebe configuration with SBT-specific values"""
-    theme_options = app.env.config.html_theme_options
+    theme_options = _get_theme_options(app)
     if theme_options.get("launch_buttons", {}).get("thebe") is True:
+        # In case somebody specifies they want thebe in a launch button
+        # but has not activated the sphinx_thebe extension.
         if not hasattr(app.env.config, "thebe_config"):
             SPHINX_LOGGER.warning(
                 (
@@ -143,6 +156,19 @@ def update_thebe_config(app):
     app.env.config.thebe_config = thebe_config
 
 
+def check_deprecation_keys(app):
+    """Warns about the deprecated keys."""
+
+    deprecated_config_list = ["single_page"]
+    for key in deprecated_config_list:
+        if key in _get_theme_options(app):
+            SPHINX_LOGGER.warning(
+                f"'{key}' was deprecated from version 0.3.4 onwards. See the CHANGELOG for more information: https://github.com/executablebooks/sphinx-book-theme/blob/master/CHANGELOG.md"  # noqa: E501
+                f"[{DEFAULT_LOG_TYPE}]",
+                type=DEFAULT_LOG_TYPE,
+            )
+
+
 class Margin(Sidebar):
     """Goes in the margin to the right of the page."""
 
@@ -162,6 +188,38 @@ class Margin(Sidebar):
         return nodes
 
 
+def update_general_config(app):
+    theme_dir = get_html_theme_path()
+    # Update templates for sidebar. Needed for jupyter-book builds as jb
+    # uses an instance of Sphinx class from sphinx.application to build the app.
+    # The __init__ function of which calls self.config.init_values() just
+    # before emitting `config-inited` event. The init_values function overwrites
+    # templates_path variable.
+    app.config.templates_path.append(os.path.join(theme_dir, "components"))
+
+
+def update_templates(app, pagename, templatename, context, doctree):
+    """Update template names and assets for page build.
+
+    This is a copy of what the pydata theme does here to include a new section
+    - https://github.com/pydata/pydata-sphinx-theme/blob/0a4894fab49befc59eb497811949a1d0ede626eb/src/pydata_sphinx_theme/__init__.py#L173 # noqa: E501
+    """
+    # Allow for more flexibility in template names
+    template_sections = ["theme_footer_content_items"]
+    for section in template_sections:
+        if context.get(section):
+            # Break apart `,` separated strings so we can use , in the defaults
+            if isinstance(context.get(section), str):
+                context[section] = [
+                    ii.strip() for ii in context.get(section).split(",")
+                ]
+
+            # Add `.html` to templates with no suffix
+            for ii, template in enumerate(context.get(section)):
+                if not os.path.splitext(template)[1]:
+                    context[section][ii] = template + ".html"
+
+
 def setup(app: Sphinx):
     # Register theme
     theme_dir = get_html_theme_path()
@@ -173,20 +231,33 @@ def setup(app: Sphinx):
     app.add_message_catalog(MESSAGE_CATALOG_NAME, locale_dir)
 
     # Events
-    app.connect("builder-inited", update_thebe_config)
+    app.connect("builder-inited", update_mode_thebe_config)
+    app.connect("builder-inited", check_deprecation_keys)
+    app.connect("builder-inited", update_sourcename)
+    app.connect("builder-inited", update_context_with_repository_info)
+    app.connect("builder-inited", update_general_config)
     app.connect("html-page-context", add_metadata_to_page)
     app.connect("html-page-context", hash_html_assets)
+    app.connect("html-page-context", update_templates)
+
+    # Nodes
+    SideNoteNode.add_node(app)
 
     # Header buttons
     app.connect("html-page-context", prep_header_buttons)
-    app.connect("html-page-context", add_launch_buttons)
-    # Bump priority by 1 so that it runs after the pydata theme sets up the edit URL.
+    # Bump priority so that it runs after the pydata theme sets up the edit URL func.
+    app.connect("html-page-context", add_launch_buttons, priority=501)
+    app.connect("html-page-context", add_source_buttons, priority=501)
     app.connect("html-page-context", add_header_buttons, priority=501)
 
     # Directives
     app.add_directive("margin", Margin)
 
-    # Update templates for sidebar
+    # Post-transforms
+    app.add_post_transform(HandleFootnoteTransform)
+
+    # Update templates for sidebar, for builds where config-inited is not called
+    # (does not work in case of jupyter-book)
     app.config.templates_path.append(os.path.join(theme_dir, "components"))
 
     return {
